@@ -667,46 +667,34 @@ int Client::init(int fd, const Address &local_addr, const Address &remote_addr,
   port_ = port;
 
   auto callbacks = ngtcp2_callbacks{
-    ngtcp2_crypto_client_initial_cb,
-    nullptr, // recv_client_initial
-    ::recv_crypto_data,
-    ::handshake_completed,
-    ::recv_version_negotiation,
-    ngtcp2_crypto_encrypt_cb,
-    ngtcp2_crypto_decrypt_cb,
-    do_hp_mask,
-    ::recv_stream_data,
-    ::acked_stream_data_offset,
-    nullptr, // stream_open
-    stream_close,
-    nullptr, // recv_stateless_reset
-    ngtcp2_crypto_recv_retry_cb,
-    extend_max_local_streams_bidi,
-    nullptr, // extend_max_local_streams_uni
-    rand,
-    get_new_connection_id,
-    nullptr, // remove_connection_id
-    ::update_key,
-    path_validation,
-    ::select_preferred_address,
-    stream_reset,
-    nullptr, // extend_max_remote_streams_bidi,
-    nullptr, // extend_max_remote_streams_uni,
-    ::extend_max_stream_data,
-    nullptr, // dcid_status
-    ::handshake_confirmed,
-    ::recv_new_token,
-    ngtcp2_crypto_delete_crypto_aead_ctx_cb,
-    ngtcp2_crypto_delete_crypto_cipher_ctx_cb,
-    nullptr, // recv_datagram
-    nullptr, // ack_datagram
-    nullptr, // lost_datagram
-    ngtcp2_crypto_get_path_challenge_data_cb,
-    stream_stop_sending,
-    ngtcp2_crypto_version_negotiation_cb,
-    ::recv_rx_key,
-    nullptr, // recv_tx_key
-    ::early_data_rejected,
+    .client_initial = ngtcp2_crypto_client_initial_cb,
+    .recv_crypto_data = ::recv_crypto_data,
+    .handshake_completed = ::handshake_completed,
+    .recv_version_negotiation = ::recv_version_negotiation,
+    .encrypt = ngtcp2_crypto_encrypt_cb,
+    .decrypt = ngtcp2_crypto_decrypt_cb,
+    .hp_mask = do_hp_mask,
+    .recv_stream_data = ::recv_stream_data,
+    .acked_stream_data_offset = ::acked_stream_data_offset,
+    .stream_close = stream_close,
+    .recv_retry = ngtcp2_crypto_recv_retry_cb,
+    .extend_max_local_streams_bidi = extend_max_local_streams_bidi,
+    .rand = rand,
+    .get_new_connection_id = get_new_connection_id,
+    .update_key = ::update_key,
+    .path_validation = path_validation,
+    .select_preferred_addr = ::select_preferred_address,
+    .stream_reset = stream_reset,
+    .extend_max_stream_data = ::extend_max_stream_data,
+    .handshake_confirmed = ::handshake_confirmed,
+    .recv_new_token = ::recv_new_token,
+    .delete_crypto_aead_ctx = ngtcp2_crypto_delete_crypto_aead_ctx_cb,
+    .delete_crypto_cipher_ctx = ngtcp2_crypto_delete_crypto_cipher_ctx_cb,
+    .get_path_challenge_data = ngtcp2_crypto_get_path_challenge_data_cb,
+    .stream_stop_sending = stream_stop_sending,
+    .version_negotiation = ngtcp2_crypto_version_negotiation_cb,
+    .recv_rx_key = ::recv_rx_key,
+    .tls_early_data_rejected = ::early_data_rejected,
   };
 
   ngtcp2_cid scid, dcid;
@@ -819,15 +807,17 @@ int Client::init(int fd, const Address &local_addr, const Address &remote_addr,
   params.grease_quic_bit = 1;
 
   auto path = ngtcp2_path{
-    {
-      const_cast<sockaddr *>(&ep.addr.su.sa),
-      ep.addr.len,
-    },
-    {
-      const_cast<sockaddr *>(&remote_addr.su.sa),
-      remote_addr.len,
-    },
-    &ep,
+    .local =
+      {
+        .addr = const_cast<sockaddr *>(&ep.addr.su.sa),
+        .addrlen = ep.addr.len,
+      },
+    .remote =
+      {
+        .addr = const_cast<sockaddr *>(&remote_addr.su.sa),
+        .addrlen = remote_addr.len,
+      },
+    .user_data = &ep,
   };
   auto rv =
     ngtcp2_conn_client_new(&conn_, &dcid, &scid, &path, client_chosen_version_,
@@ -874,15 +864,17 @@ int Client::feed_data(const Endpoint &ep, const sockaddr *sa, socklen_t salen,
                       const ngtcp2_pkt_info *pi,
                       std::span<const uint8_t> data) {
   auto path = ngtcp2_path{
-    {
-      const_cast<sockaddr *>(&ep.addr.su.sa),
-      ep.addr.len,
-    },
-    {
-      const_cast<sockaddr *>(sa),
-      salen,
-    },
-    const_cast<Endpoint *>(&ep),
+    .local =
+      {
+        .addr = const_cast<sockaddr *>(&ep.addr.su.sa),
+        .addrlen = ep.addr.len,
+      },
+    .remote =
+      {
+        .addr = const_cast<sockaddr *>(sa),
+        .addrlen = salen,
+      },
+    .user_data = const_cast<Endpoint *>(&ep),
   };
   if (auto rv = ngtcp2_conn_read_pkt(conn_, &path, pi, data.data(), data.size(),
                                      util::timestamp());
@@ -908,17 +900,19 @@ int Client::on_read(const Endpoint &ep) {
   size_t pktcnt = 0;
   ngtcp2_pkt_info pi;
 
-  iovec msg_iov;
-  msg_iov.iov_base = buf.data();
-  msg_iov.iov_len = buf.size();
-
-  msghdr msg{};
-  msg.msg_name = &su;
-  msg.msg_iov = &msg_iov;
-  msg.msg_iovlen = 1;
+  iovec msg_iov{
+    .iov_base = buf.data(),
+    .iov_len = buf.size(),
+  };
 
   uint8_t msg_ctrl[CMSG_SPACE(sizeof(int)) + CMSG_SPACE(sizeof(int))];
-  msg.msg_control = msg_ctrl;
+
+  msghdr msg{
+    .msg_name = &su,
+    .msg_iov = &msg_iov,
+    .msg_iovlen = 1,
+    .msg_control = msg_ctrl,
+  };
 
   for (;;) {
     msg.msg_namelen = sizeof(su);
@@ -933,8 +927,8 @@ int Client::on_read(const Endpoint &ep) {
       break;
     }
 
-    // Packets less than 22 bytes never be a valid QUIC packet.
-    if (nread < 22) {
+    // Packets less than 21 bytes never be a valid QUIC packet.
+    if (nread < 21) {
       ++pktcnt;
 
       continue;
@@ -961,8 +955,8 @@ int Client::on_read(const Endpoint &ep) {
                   << std::dec << " " << datalen << " bytes" << std::endl;
       }
 
-      // Packets less than 22 bytes never be a valid QUIC packet.
-      if (datalen < 22) {
+      // Packets less than 21 bytes never be a valid QUIC packet.
+      if (datalen < 21) {
         break;
       }
 
@@ -1248,13 +1242,12 @@ void Client::update_timer() {
 namespace {
 int bind_addr(Address &local_addr, int fd, const in_addr_union *iau,
               int family) {
-  addrinfo hints{};
+  addrinfo hints{
+    .ai_flags = AI_PASSIVE,
+    .ai_family = family,
+    .ai_socktype = SOCK_DGRAM,
+  };
   addrinfo *res, *rp;
-
-  hints.ai_family = family;
-  hints.ai_socktype = SOCK_DGRAM;
-  hints.ai_flags = AI_PASSIVE;
-
   char *node;
   std::array<char, NI_MAXHOST> nodebuf;
 
@@ -1339,12 +1332,12 @@ int udp_sock(int family) {
 
 namespace {
 int create_sock(Address &remote_addr, const char *addr, const char *port) {
-  addrinfo hints{};
+  addrinfo hints{
+    .ai_family = AF_UNSPEC,
+    .ai_socktype = SOCK_DGRAM,
+  };
   addrinfo *res, *rp;
 
-  hints.ai_family = AF_UNSPEC;
-  hints.ai_socktype = SOCK_DGRAM;
-
   if (auto rv = getaddrinfo(addr, port, &hints, &res); rv != 0) {
     std::cerr << "getaddrinfo: " << gai_strerror(rv) << std::endl;
     return -1;
@@ -1482,12 +1475,13 @@ int Client::change_local_addr() {
     ngtcp2_conn_set_path_user_data(conn_, &ep);
   } else {
     auto path = ngtcp2_path{
-      addr,
-      {
-        const_cast<sockaddr *>(&remote_addr_.su.sa),
-        remote_addr_.len,
-      },
-      &ep,
+      .local = addr,
+      .remote =
+        {
+          .addr = const_cast<sockaddr *>(&remote_addr_.su.sa),
+          .addrlen = remote_addr_.len,
+        },
+      .user_data = &ep,
     };
     if (auto rv = ngtcp2_conn_initiate_immediate_migration(conn_, &path,
                                                            util::timestamp());
@@ -1598,24 +1592,23 @@ Client::send_packet(const Endpoint &ep, const ngtcp2_addr &remote_addr,
     return {{}, 0};
   }
 
-  iovec msg_iov;
-  msg_iov.iov_base = const_cast<uint8_t *>(data.data());
-  msg_iov.iov_len = data.size();
+  iovec msg_iov{
+    .iov_base = const_cast<uint8_t *>(data.data()),
+    .iov_len = data.size(),
+  };
 
-  msghdr msg{};
+  uint8_t msg_ctrl[CMSG_SPACE(sizeof(int)) + CMSG_SPACE(sizeof(uint16_t))]{};
+
+  msghdr msg{
 #ifdef HAVE_LINUX_RTNETLINK_H
-  msg.msg_name = const_cast<sockaddr *>(remote_addr.addr);
-  msg.msg_namelen = remote_addr.addrlen;
+    .msg_name = const_cast<sockaddr *>(remote_addr.addr),
+    .msg_namelen = remote_addr.addrlen,
 #endif // defined(HAVE_LINUX_RTNETLINK_H)
-  msg.msg_iov = &msg_iov;
-  msg.msg_iovlen = 1;
-
-  uint8_t msg_ctrl[CMSG_SPACE(sizeof(int)) + CMSG_SPACE(sizeof(uint16_t))];
-
-  memset(msg_ctrl, 0, sizeof(msg_ctrl));
-
-  msg.msg_control = msg_ctrl;
-  msg.msg_controllen = sizeof(msg_ctrl);
+    .msg_iov = &msg_iov,
+    .msg_iovlen = 1,
+    .msg_control = msg_ctrl,
+    .msg_controllen = sizeof(msg_ctrl),
+  };
 
   size_t controllen = 0;
 
@@ -1929,8 +1922,9 @@ int Client::submit_http_request(const Stream *stream) {
     debug::print_http_request_headers(stream->stream_id, nva.data(), nvlen);
   }
 
-  nghttp3_data_reader dr{};
-  dr.read_data = read_data;
+  nghttp3_data_reader dr{
+    .read_data = read_data,
+  };
 
   if (auto rv = nghttp3_conn_submit_request(
         httpconn_, stream->stream_id, nva.data(), nvlen,
@@ -2225,21 +2219,18 @@ int Client::setup_httpconn() {
   }
 
   nghttp3_callbacks callbacks{
-    nullptr, // acked_stream_data
-    ::http_stream_close,
-    ::http_recv_data,
-    ::http_deferred_consume,
-    ::http_begin_headers,
-    ::http_recv_header,
-    ::http_end_headers,
-    ::http_begin_trailers,
-    ::http_recv_trailer,
-    ::http_end_trailers,
-    ::http_stop_sending,
-    nullptr, // end_stream
-    ::http_reset_stream,
-    nullptr, // shutdown
-    ::http_recv_settings,
+    .stream_close = ::http_stream_close,
+    .recv_data = ::http_recv_data,
+    .deferred_consume = ::http_deferred_consume,
+    .begin_headers = ::http_begin_headers,
+    .recv_header = ::http_recv_header,
+    .end_headers = ::http_end_headers,
+    .begin_trailers = ::http_begin_trailers,
+    .recv_trailer = ::http_recv_trailer,
+    .end_trailers = ::http_end_trailers,
+    .stop_sending = ::http_stop_sending,
+    .reset_stream = ::http_reset_stream,
+    .recv_settings = ::http_recv_settings,
   };
   nghttp3_settings settings;
   nghttp3_settings_default(&settings);
@@ -2444,27 +2435,25 @@ void print_usage() {
 
 namespace {
 void config_set_default(Config &config) {
-  config = Config{};
-  config.tx_loss_prob = 0.;
-  config.rx_loss_prob = 0.;
-  config.fd = -1;
-  config.ciphers = util::crypto_default_ciphers();
-  config.groups = util::crypto_default_groups();
-  config.nstreams = 0;
-  config.data = nullptr;
-  config.datalen = 0;
-  config.version = NGTCP2_PROTO_VER_V1;
-  config.timeout = 30 * NGTCP2_SECONDS;
-  config.http_method = "GET"sv;
-  config.max_data = 24_m;
-  config.max_stream_data_bidi_local = 16_m;
-  config.max_stream_data_uni = 16_m;
-  config.max_streams_uni = 100;
-  config.cc_algo = NGTCP2_CC_ALGO_CUBIC;
-  config.initial_rtt = NGTCP2_DEFAULT_INITIAL_RTT;
-  config.handshake_timeout = UINT64_MAX;
-  config.ack_thresh = 2;
-  config.initial_pkt_num = UINT32_MAX;
+  config = Config{
+    .tx_loss_prob = 0.,
+    .rx_loss_prob = 0.,
+    .fd = -1,
+    .ciphers = util::crypto_default_ciphers(),
+    .groups = util::crypto_default_groups(),
+    .version = NGTCP2_PROTO_VER_V1,
+    .timeout = 30 * NGTCP2_SECONDS,
+    .http_method = "GET"sv,
+    .max_data = 24_m,
+    .max_stream_data_bidi_local = 16_m,
+    .max_stream_data_uni = 16_m,
+    .max_streams_uni = 100,
+    .cc_algo = NGTCP2_CC_ALGO_CUBIC,
+    .initial_rtt = NGTCP2_DEFAULT_INITIAL_RTT,
+    .handshake_timeout = UINT64_MAX,
+    .ack_thresh = 2,
+    .initial_pkt_num = UINT32_MAX,
+  };
 }
 } // namespace
 
@@ -2763,7 +2752,7 @@ int main(int argc, char **argv) {
       {"wait-for-ticket", no_argument, &flag, 41},
       {"initial-pkt-num", required_argument, &flag, 42},
       {"pmtud-probes", required_argument, &flag, 43},
-      {nullptr, 0, nullptr, 0},
+      {},
     };
 
     auto optidx = 0;
