@@ -27,7 +27,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif // HAVE_CONFIG_H
+#endif // defined(HAVE_CONFIG_H)
 
 #include <vector>
 #include <deque>
@@ -35,6 +35,7 @@
 #include <string_view>
 #include <memory>
 #include <set>
+#include <span>
 
 #include <ngtcp2/ngtcp2.h>
 #include <ngtcp2/ngtcp2_crypto.h>
@@ -94,7 +95,7 @@ public:
   int on_write();
   int write_streams();
   int feed_data(const Endpoint &ep, const sockaddr *sa, socklen_t salen,
-                const ngtcp2_pkt_info *pi, uint8_t *data, size_t datalen);
+                const ngtcp2_pkt_info *pi, std::span<const uint8_t> data);
   int handle_expiry();
   void update_timer();
   int handshake_completed();
@@ -102,7 +103,10 @@ public:
   void recv_version_negotiation(const uint32_t *sv, size_t nsv);
 
   int send_packet(const Endpoint &ep, const ngtcp2_addr &remote_addr,
-                  unsigned int ecn, const uint8_t *data, size_t datalen);
+                  unsigned int ecn, std::span<const uint8_t> data);
+  std::pair<std::span<const uint8_t>, int>
+  send_packet(const Endpoint &ep, const ngtcp2_addr &remote_addr,
+              unsigned int ecn, std::span<const uint8_t> data, size_t gso_size);
   int on_stream_close(int64_t stream_id, uint64_t app_error_code);
   int on_extend_max_streams();
   int handle_error();
@@ -126,8 +130,8 @@ public:
   void set_remote_addr(const ngtcp2_addr &remote_addr);
 
   int submit_http_request(Stream *stream);
-  int recv_stream_data(uint32_t flags, int64_t stream_id, const uint8_t *data,
-                       size_t datalen);
+  int recv_stream_data(uint32_t flags, int64_t stream_id,
+                       std::span<const uint8_t> data);
   int acked_stream_data_offset(int64_t stream_id, uint64_t offset,
                                uint64_t datalen);
   int extend_max_stream_data(int64_t stream_id, uint64_t max_data);
@@ -135,7 +139,8 @@ public:
   void write_qlog(const void *data, size_t datalen);
 
   void on_send_blocked(const Endpoint &ep, const ngtcp2_addr &remote_addr,
-                       unsigned int ecn, size_t datalen);
+                       unsigned int ecn, std::span<const uint8_t> data,
+                       size_t gso_size);
   void start_wev_endpoint(const Endpoint &ep);
   int send_blocked_packet();
 
@@ -143,6 +148,8 @@ public:
 
   void early_data_rejected();
 
+  bool should_exit() const;
+
 private:
   std::vector<Endpoint> endpoints_;
   Address remote_addr_;
@@ -170,27 +177,25 @@ private:
   uint32_t original_version_;
   // early_data_ is true if client attempts to do 0RTT data transfer.
   bool early_data_;
-  // should_exit_ is true if client should exit rather than waiting
-  // for timeout.
-  bool should_exit_;
-  // should_exit_on_handshake_confirmed_ is true if client should exit
-  // when handshake confirmed.
-  bool should_exit_on_handshake_confirmed_;
   // handshake_confirmed_ gets true after handshake has been
   // confirmed.
   bool handshake_confirmed_;
+  bool no_gso_;
 
   struct {
     bool send_blocked;
+    size_t num_blocked;
+    size_t num_blocked_sent;
     // blocked field is effective only when send_blocked is true.
     struct {
       const Endpoint *endpoint;
       Address remote_addr;
       unsigned int ecn;
-      size_t datalen;
-    } blocked;
+      std::span<const uint8_t> data;
+      size_t gso_size;
+    } blocked[2];
     std::array<uint8_t, 64_k> data;
   } tx_;
 };
 
-#endif // CLIENT_H
+#endif // !defined(H09CLIENT_H)
