@@ -22,12 +22,12 @@
  * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-#ifndef SERVER_H
-#define SERVER_H
+#ifndef H09SERVER_H
+#define H09SERVER_H
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif // HAVE_CONFIG_H
+#endif // defined(HAVE_CONFIG_H)
 
 #include <vector>
 #include <unordered_map>
@@ -36,6 +36,7 @@
 #include <string_view>
 #include <memory>
 #include <set>
+#include <span>
 
 #include <ngtcp2/ngtcp2.h>
 #include <ngtcp2/ngtcp2_crypto.h>
@@ -47,12 +48,13 @@
 #include "tls_server_context.h"
 #include "network.h"
 #include "shared.h"
+#include "util.h"
 
 using namespace ngtcp2;
 
 struct HTTPHeader {
   HTTPHeader(const std::string_view &name, const std::string_view &value)
-      : name(name), value(value) {}
+    : name(name), value(value) {}
 
   std::string_view name;
   std::string_view value;
@@ -94,8 +96,6 @@ struct Endpoint {
   ev_io rev;
   Server *server;
   int fd;
-  // ecn is the last ECN bits set to fd.
-  unsigned int ecn;
 };
 
 class Handler : public HandlerBase {
@@ -105,25 +105,26 @@ public:
 
   int init(const Endpoint &ep, const Address &local_addr, const sockaddr *sa,
            socklen_t salen, const ngtcp2_cid *dcid, const ngtcp2_cid *scid,
-           const ngtcp2_cid *ocid, const uint8_t *token, size_t tokenlen,
-           uint32_t version, TLSServerContext &tls_ctx);
+           const ngtcp2_cid *ocid, std::span<const uint8_t> token,
+           ngtcp2_token_type token_type, uint32_t version,
+           TLSServerContext &tls_ctx);
 
   int on_read(const Endpoint &ep, const Address &local_addr, const sockaddr *sa,
-              socklen_t salen, const ngtcp2_pkt_info *pi, const uint8_t *data,
-              size_t datalen);
+              socklen_t salen, const ngtcp2_pkt_info *pi,
+              std::span<const uint8_t> data);
   int on_write();
   int write_streams();
   int feed_data(const Endpoint &ep, const Address &local_addr,
                 const sockaddr *sa, socklen_t salen, const ngtcp2_pkt_info *pi,
-                const uint8_t *data, size_t datalen);
+                std::span<const uint8_t> data);
   void update_timer();
   int handle_expiry();
   void signal_write();
   int handshake_completed();
 
   Server *server() const;
-  int recv_stream_data(uint32_t flags, int64_t stream_id, const uint8_t *data,
-                       size_t datalen);
+  int recv_stream_data(uint32_t flags, int64_t stream_id,
+                       std::span<const uint8_t> data);
   int acked_stream_data_offset(int64_t stream_id, uint64_t offset,
                                uint64_t datalen);
   uint32_t version() const;
@@ -149,7 +150,7 @@ public:
 
   void on_send_blocked(Endpoint &ep, const ngtcp2_addr &local_addr,
                        const ngtcp2_addr &remote_addr, unsigned int ecn,
-                       const uint8_t *data, size_t datalen, size_t gso_size);
+                       std::span<const uint8_t> data, size_t gso_size);
   void start_wev_endpoint(const Endpoint &ep);
   int send_blocked_packet();
 
@@ -180,8 +181,7 @@ private:
       Address local_addr;
       Address remote_addr;
       unsigned int ecn;
-      const uint8_t *data;
-      size_t datalen;
+      std::span<const uint8_t> data;
       size_t gso_size;
     } blocked[2];
     std::unique_ptr<uint8_t[]> data;
@@ -199,11 +199,10 @@ public:
 
   int on_read(Endpoint &ep);
   void read_pkt(Endpoint &ep, const Address &local_addr, const sockaddr *sa,
-                socklen_t salen, const ngtcp2_pkt_info *pi, const uint8_t *data,
-                size_t datalen);
-  int send_version_negotiation(uint32_t version, const uint8_t *dcid,
-                               size_t dcidlen, const uint8_t *scid,
-                               size_t scidlen, Endpoint &ep,
+                socklen_t salen, const ngtcp2_pkt_info *pi,
+                std::span<const uint8_t> data);
+  int send_version_negotiation(uint32_t version, std::span<const uint8_t> dcid,
+                               std::span<const uint8_t> scid, Endpoint &ep,
                                const Address &local_addr, const sockaddr *sa,
                                socklen_t salen);
   int send_retry(const ngtcp2_pkt_hd *chd, Endpoint &ep,
@@ -212,29 +211,35 @@ public:
   int send_stateless_connection_close(const ngtcp2_pkt_hd *chd, Endpoint &ep,
                                       const Address &local_addr,
                                       const sockaddr *sa, socklen_t salen);
+  int send_stateless_reset(size_t pktlen, std::span<const uint8_t> dcid,
+                           Endpoint &ep, const Address &local_addr,
+                           const sockaddr *sa, socklen_t salen);
   int verify_retry_token(ngtcp2_cid *ocid, const ngtcp2_pkt_hd *hd,
                          const sockaddr *sa, socklen_t salen);
   int verify_token(const ngtcp2_pkt_hd *hd, const sockaddr *sa,
                    socklen_t salen);
   int send_packet(Endpoint &ep, const ngtcp2_addr &local_addr,
                   const ngtcp2_addr &remote_addr, unsigned int ecn,
-                  const uint8_t *data, size_t datalen);
-  std::pair<size_t, int> send_packet(Endpoint &ep, bool &no_gso,
-                                     const ngtcp2_addr &local_addr,
-                                     const ngtcp2_addr &remote_addr,
-                                     unsigned int ecn, const uint8_t *data,
-                                     size_t datalen, size_t gso_size);
+                  std::span<const uint8_t> data);
+  std::pair<std::span<const uint8_t>, int>
+  send_packet(Endpoint &ep, bool &no_gso, const ngtcp2_addr &local_addr,
+              const ngtcp2_addr &remote_addr, unsigned int ecn,
+              std::span<const uint8_t> data, size_t gso_size);
   void remove(const Handler *h);
 
   void associate_cid(const ngtcp2_cid *cid, Handler *h);
   void dissociate_cid(const ngtcp2_cid *cid);
 
+  void on_stateless_reset_regen();
+
 private:
-  std::unordered_map<std::string, Handler *> handlers_;
+  std::unordered_map<ngtcp2_cid, Handler *> handlers_;
   struct ev_loop *loop_;
   std::vector<Endpoint> endpoints_;
   TLSServerContext &tls_ctx_;
   ev_signal sigintev_;
+  ev_timer stateless_reset_regen_timer_;
+  size_t stateless_reset_bucket_;
 };
 
-#endif // SERVER_H
+#endif // !defined(H09SERVER_H)
