@@ -27,7 +27,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif // HAVE_CONFIG_H
+#endif // defined(HAVE_CONFIG_H)
 
 #include <vector>
 #include <deque>
@@ -35,6 +35,7 @@
 #include <string>
 #include <string_view>
 #include <functional>
+#include <span>
 
 #include <ngtcp2/ngtcp2_crypto.h>
 
@@ -132,9 +133,6 @@ struct Config {
   // send_trailers controls whether server sends trailer fields or
   // not.
   bool send_trailers;
-  // max_gso_dgrams is the maximum number of UDP datagrams in one GSO
-  // sendmsg call.
-  size_t max_gso_dgrams;
   // handshake_timeout is the period of time before giving up QUIC
   // connection establishment.
   ngtcp2_duration handshake_timeout;
@@ -165,7 +163,7 @@ struct Buffer {
   size_t size() const { return tail - begin; }
   size_t left() const { return buf.data() + buf.size() - tail; }
   uint8_t *const wpos() { return tail; }
-  const uint8_t *rpos() const { return begin; }
+  std::span<const uint8_t> data() const { return {begin, size()}; }
   void push(size_t len) { tail += len; }
   void reset() { tail = begin; }
 
@@ -197,4 +195,4 @@ protected:
   ngtcp2_ccerr last_error_;
 };
 
-#endif // SERVER_BASE_H
+#endif // !defined(SERVER_BASE_H)
