@@ -29,12 +29,13 @@
 #include <functional>
 #include <utility>
 #include <type_traits>
+#include <span>
 
 // inspired by <http://blog.korfuri.fr/post/go-defer-in-cpp/>, but our
 // template can take functions returning other than void.
 template <typename F, typename... T> struct Defer {
   Defer(F &&f, T &&...t)
-      : f(std::bind(std::forward<F>(f), std::forward<T>(t)...)) {}
+    : f(std::bind(std::forward<F>(f), std::forward<T>(t)...)) {}
   Defer(Defer &&o) noexcept : f(std::move(o.f)) {}
   ~Defer() { f(); }
 
@@ -68,4 +69,15 @@ constexpr unsigned long long operator""_g(unsigned long long g) {
   return g * 1024 * 1024 * 1024;
 }
 
-#endif // TEMPLATE_H
+template <typename T, std::size_t N>
+[[nodiscard]] std::span<uint8_t, N == std::dynamic_extent ? std::dynamic_extent
+                                                          : N * sizeof(T)>
+as_writable_uint8_span(std::span<T, N> s) noexcept {
+  return std::span < uint8_t,
+         N == std::dynamic_extent
+           ? std::dynamic_extent
+           : N * sizeof(T) >
+               {reinterpret_cast<uint8_t *>(s.data()), s.size_bytes()};
+}
+
+#endif // !defined(TEMPLATE_H)
