@@ -24,6 +24,7 @@
  */
 #include "tls_client_session_picotls.h"
 
+#include <cstring>
 #include <iostream>
 #include <memory>
 
@@ -52,12 +53,21 @@ TLSClientSession::~TLSClientSession() {
 }
 
 namespace {
-auto negotiated_protocols = std::array<ptls_iovec_t, 1>{{
-    {
-        .base = const_cast<uint8_t *>(&H3_ALPN_V1[1]),
-        .len = H3_ALPN_V1[0],
-    },
-}};
+auto negotiated_protocols_h3 = std::to_array<ptls_iovec_t>({
+  {
+    .base = const_cast<uint8_t *>(&H3_ALPN_V1[1]),
+    .len = H3_ALPN_V1[0],
+  },
+});
+} // namespace
+
+namespace {
+auto negotiated_protocols_hq = std::to_array<ptls_iovec_t>({
+  {
+    .base = const_cast<uint8_t *>(&HQ_ALPN_V1[1]),
+    .len = HQ_ALPN_V1[0],
+  },
+});
 } // namespace
 
 int TLSClientSession::init(bool &early_data_enabled, TLSClientContext &tls_ctx,
@@ -75,12 +85,12 @@ int TLSClientSession::init(bool &early_data_enabled, TLSClientContext &tls_ctx,
   auto &hsprops = cptls_.handshake_properties;
 
   hsprops.additional_extensions = new ptls_raw_extension_t[2]{
-      {
-          .type = UINT16_MAX,
-      },
-      {
-          .type = UINT16_MAX,
-      },
+    {
+      .type = UINT16_MAX,
+    },
+    {
+      .type = UINT16_MAX,
+    },
   };
 
   if (ngtcp2_crypto_picotls_configure_client_session(&cptls_, conn) != 0) {
@@ -89,8 +99,18 @@ int TLSClientSession::init(bool &early_data_enabled, TLSClientContext &tls_ctx,
     return -1;
   }
 
-  hsprops.client.negotiated_protocols.list = negotiated_protocols.data();
-  hsprops.client.negotiated_protocols.count = negotiated_protocols.size();
+  switch (app_proto) {
+  case AppProtocol::H3:
+    hsprops.client.negotiated_protocols.list = negotiated_protocols_h3.data();
+    hsprops.client.negotiated_protocols.count = negotiated_protocols_h3.size();
+
+    break;
+  case AppProtocol::HQ:
+    hsprops.client.negotiated_protocols.list = negotiated_protocols_hq.data();
+    hsprops.client.negotiated_protocols.count = negotiated_protocols_hq.size();
+
+    break;
+  }
 
   if (util::numeric_host(remote_addr)) {
     // If remote host is numeric address, just send "localhost" as SNI
