@@ -24,6 +24,7 @@
  */
 #include "tls_server_context_boringssl.h"
 
+#include <cstring>
 #include <iostream>
 #include <fstream>
 
@@ -33,6 +34,7 @@
 
 #include "server_base.h"
 #include "template.h"
+#include "tls_shared_boringssl.h"
 
 extern Config config;
 
@@ -59,24 +61,8 @@ int alpn_select_proto_h3_cb(SSL *ssl, const unsigned char **out,
   auto version = ngtcp2_conn_get_client_chosen_version(h->conn());
 
   switch (version) {
-  case QUIC_VER_DRAFT29:
-    alpn = H3_ALPN_DRAFT29;
-    alpnlen = str_size(H3_ALPN_DRAFT29);
-    break;
-  case QUIC_VER_DRAFT30:
-    alpn = H3_ALPN_DRAFT30;
-    alpnlen = str_size(H3_ALPN_DRAFT30);
-    break;
-  case QUIC_VER_DRAFT31:
-    alpn = H3_ALPN_DRAFT31;
-    alpnlen = str_size(H3_ALPN_DRAFT31);
-    break;
-  case QUIC_VER_DRAFT32:
-    alpn = H3_ALPN_DRAFT32;
-    alpnlen = str_size(H3_ALPN_DRAFT32);
-    break;
   case NGTCP2_PROTO_VER_V1:
-  case NGTCP2_PROTO_VER_V2_DRAFT:
+  case NGTCP2_PROTO_VER_V2:
     alpn = H3_ALPN_V1;
     alpnlen = str_size(H3_ALPN_V1);
     break;
@@ -117,24 +103,8 @@ int alpn_select_proto_hq_cb(SSL *ssl, const unsigned char **out,
   auto version = ngtcp2_conn_get_client_chosen_version(h->conn());
 
   switch (version) {
-  case QUIC_VER_DRAFT29:
-    alpn = HQ_ALPN_DRAFT29;
-    alpnlen = str_size(HQ_ALPN_DRAFT29);
-    break;
-  case QUIC_VER_DRAFT30:
-    alpn = HQ_ALPN_DRAFT30;
-    alpnlen = str_size(HQ_ALPN_DRAFT30);
-    break;
-  case QUIC_VER_DRAFT31:
-    alpn = HQ_ALPN_DRAFT31;
-    alpnlen = str_size(HQ_ALPN_DRAFT31);
-    break;
-  case QUIC_VER_DRAFT32:
-    alpn = HQ_ALPN_DRAFT32;
-    alpnlen = str_size(HQ_ALPN_DRAFT32);
-    break;
   case NGTCP2_PROTO_VER_V1:
-  case NGTCP2_PROTO_VER_V2_DRAFT:
+  case NGTCP2_PROTO_VER_V2:
     alpn = HQ_ALPN_V1;
     alpnlen = str_size(HQ_ALPN_V1);
     break;
@@ -187,8 +157,8 @@ int TLSServerContext::init(const char *private_key_file, const char *cert_file,
 
   SSL_CTX_set_options(ssl_ctx_, ssl_opts);
 
-  if (SSL_CTX_set1_curves_list(ssl_ctx_, config.groups) != 1) {
-    std::cerr << "SSL_CTX_set1_curves_list failed" << std::endl;
+  if (SSL_CTX_set1_groups_list(ssl_ctx_, config.groups) != 1) {
+    std::cerr << "SSL_CTX_set1_groups_list failed" << std::endl;
     return -1;
   }
 
@@ -235,10 +205,19 @@ int TLSServerContext::init(const char *private_key_file, const char *cert_file,
   if (config.verify_client) {
     SSL_CTX_set_verify(ssl_ctx_,
                        SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE |
-                           SSL_VERIFY_FAIL_IF_NO_PEER_CERT,
+                         SSL_VERIFY_FAIL_IF_NO_PEER_CERT,
                        verify_cb);
   }
 
+#ifdef HAVE_LIBBROTLI
+  if (!SSL_CTX_add_cert_compression_alg(
+        ssl_ctx_, ngtcp2::tls::CERTIFICATE_COMPRESSION_ALGO_BROTLI,
+        ngtcp2::tls::cert_compress, ngtcp2::tls::cert_decompress)) {
+    std::cerr << "SSL_CTX_add_cert_compression_alg failed" << std::endl;
+    return -1;
+  }
+#endif // defined(HAVE_LIBBROTLI)
+
   return 0;
 }
 
