@@ -27,10 +27,10 @@
 
 #ifdef HAVE_ARPA_INET_H
 #  include <arpa/inet.h>
-#endif // HAVE_ARPA_INET_H
+#endif // defined(HAVE_ARPA_INET_H)
 #ifdef HAVE_NETINET_IN_H
 #  include <netinet/in.h>
-#endif
+#endif // defined(HAVE_NETINET_IN_H)
 #include <sys/types.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -54,6 +54,13 @@ namespace ngtcp2 {
 
 namespace util {
 
+std::optional<std::string> read_pem(const std::string_view &filename,
+                                    const std::string_view &name,
+                                    const std::string_view &type);
+
+int write_pem(const std::string_view &filename, const std::string_view &name,
+              const std::string_view &type, std::span<const uint8_t> data);
+
 namespace {
 constexpr char LOWER_XDIGITS[] = "0123456789abcdef";
 } // namespace
@@ -68,21 +75,21 @@ std::string format_hex(uint8_t c) {
   return s;
 }
 
-std::string format_hex(const uint8_t *s, size_t len) {
+std::string format_hex(std::span<const uint8_t> s) {
   std::string res;
-  res.resize(len * 2);
+  res.resize(s.size() * 2);
 
-  for (size_t i = 0; i < len; ++i) {
-    auto c = s[i];
+  auto p = std::begin(res);
 
-    res[i * 2] = LOWER_XDIGITS[c >> 4];
-    res[i * 2 + 1] = LOWER_XDIGITS[c & 0x0f];
+  for (auto c : s) {
+    *p++ = LOWER_XDIGITS[c >> 4];
+    *p++ = LOWER_XDIGITS[c & 0x0f];
   }
   return res;
 }
 
 std::string format_hex(const std::string_view &s) {
-  return format_hex(reinterpret_cast<const uint8_t *>(s.data()), s.size());
+  return format_hex({reinterpret_cast<const uint8_t *>(s.data()), s.size()});
 }
 
 std::string decode_hex(const std::string_view &s) {
@@ -161,10 +168,10 @@ std::mt19937 make_mt19937() {
   return std::mt19937(rd());
 }
 
-ngtcp2_tstamp timestamp(struct ev_loop *loop) {
+ngtcp2_tstamp timestamp() {
   return std::chrono::duration_cast<std::chrono::nanoseconds>(
-             std::chrono::steady_clock::now().time_since_epoch())
-      .count();
+           std::chrono::steady_clock::now().time_since_epoch())
+    .count();
 }
 
 bool numeric_host(const char *hostname) {
@@ -181,74 +188,176 @@ bool numeric_host(const char *hostname, int family) {
 }
 
 namespace {
-void hexdump8(FILE *out, const uint8_t *first, const uint8_t *last) {
-  auto stop = std::min(first + 8, last);
-  for (auto k = first; k != stop; ++k) {
-    fprintf(out, "%02x ", *k);
+uint8_t *hexdump_addr(uint8_t *dest, size_t addr) {
+  // Lower 32 bits are displayed.
+  for (size_t i = 0; i < 4; ++i) {
+    auto a = (addr >> (3 - i) * 8) & 0xff;
+
+    *dest++ = LOWER_XDIGITS[a >> 4];
+    *dest++ = LOWER_XDIGITS[a & 0xf];
   }
-  // each byte needs 3 spaces (2 hex value and space)
-  for (; stop != first + 8; ++stop) {
-    fputs("   ", out);
-  }
-  // we have extra space after 8 bytes
-  fputc(' ', out);
+
+  return dest;
 }
 } // namespace
 
-void hexdump(FILE *out, const uint8_t *src, size_t len) {
-  if (len == 0) {
-    return;
+namespace {
+uint8_t *hexdump_ascii(uint8_t *dest, std::span<const uint8_t> data) {
+  *dest++ = '|';
+
+  for (auto c : data) {
+    if (0x20 <= c && c <= 0x7e) {
+      *dest++ = c;
+    } else {
+      *dest++ = '.';
+    }
   }
-  size_t buflen = 0;
+
+  *dest++ = '|';
+
+  return dest;
+}
+} // namespace
+
+namespace {
+uint8_t *hexdump8(uint8_t *dest, std::span<const uint8_t> data) {
+  for (auto c : data) {
+    *dest++ = LOWER_XDIGITS[c >> 4];
+    *dest++ = LOWER_XDIGITS[c & 0xf];
+    *dest++ = ' ';
+  }
+
+  for (auto i = data.size(); i < 8; ++i) {
+    *dest++ = ' ';
+    *dest++ = ' ';
+    *dest++ = ' ';
+  }
+
+  return dest;
+}
+} // namespace
+
+namespace {
+uint8_t *hexdump16(uint8_t *dest, std::span<const uint8_t> data) {
+  if (data.size() > 8) {
+    dest = hexdump8(dest, {data.data(), 8});
+    *dest++ = ' ';
+    dest = hexdump8(dest, data.subspan(8));
+    *dest++ = ' ';
+  } else {
+    dest = hexdump8(dest, data);
+    *dest++ = ' ';
+    dest = hexdump8(dest, {});
+    *dest++ = ' ';
+  }
+
+  return dest;
+}
+} // namespace
+
+namespace {
+uint8_t *hexdump_line(uint8_t *dest, std::span<const uint8_t> data,
+                      size_t addr) {
+  dest = hexdump_addr(dest, addr);
+  *dest++ = ' ';
+  *dest++ = ' ';
+
+  dest = hexdump16(dest, data);
+
+  return hexdump_ascii(dest, data);
+}
+} // namespace
+
+namespace {
+int hexdump_write(int fd, std::span<const uint8_t> data) {
+  ssize_t nwrite;
+
+  for (;
+       (nwrite = write(fd, data.data(), data.size())) == -1 && errno == EINTR;)
+    ;
+  if (nwrite == -1) {
+    return -1;
+  }
+
+  return 0;
+}
+} // namespace
+
+int hexdump(FILE *out, const void *data, size_t datalen) {
+  if (datalen == 0) {
+    return 0;
+  }
+
+  // min_space is the additional minimum space that the buffer must
+  // accept, which is the size of a single full line output + one
+  // repeat line marker ("*\n").  If the remaining buffer size is less
+  // than that, flush the buffer and reset.
+  constexpr size_t min_space = 79 + 2;
+
+  auto fd = fileno(out);
+  std::array<uint8_t, 4096> buf;
+  auto last = buf.data();
+  auto in = reinterpret_cast<const uint8_t *>(data);
   auto repeated = false;
-  std::array<uint8_t, 16> buf{};
-  auto end = src + len;
-  auto i = src;
-  for (;;) {
-    auto nextlen =
-        std::min(static_cast<size_t>(16), static_cast<size_t>(end - i));
-    if (nextlen == buflen &&
-        std::equal(std::begin(buf), std::begin(buf) + buflen, i)) {
-      // as long as adjacent 16 bytes block are the same, we just
-      // print single '*'.
-      if (!repeated) {
-        repeated = true;
-        fputs("*\n", out);
-      }
-      i += nextlen;
-      continue;
-    }
-    repeated = false;
-    fprintf(out, "%08lx", static_cast<unsigned long>(i - src));
-    if (i == end) {
-      fputc('\n', out);
-      break;
-    }
-    fputs("  ", out);
-    hexdump8(out, i, end);
-    hexdump8(out, i + 8, std::max(i + 8, end));
-    fputc('|', out);
-    auto stop = std::min(i + 16, end);
-    buflen = stop - i;
-    auto p = buf.data();
-    for (; i != stop; ++i) {
-      *p++ = *i;
-      if (0x20 <= *i && *i <= 0x7e) {
-        fputc(*i, out);
-      } else {
-        fputc('.', out);
+
+  for (size_t offset = 0; offset < datalen; offset += 16) {
+    auto n = datalen - offset;
+    auto s = in + offset;
+
+    if (n >= 16) {
+      n = 16;
+
+      if (offset > 0) {
+        if (std::equal(s - 16, s, s)) {
+          if (repeated) {
+            continue;
+          }
+
+          repeated = true;
+
+          *last++ = '*';
+          *last++ = '\n';
+
+          continue;
+        }
+
+        repeated = false;
       }
     }
-    fputs("|\n", out);
+
+    last = hexdump_line(last, {s, n}, offset);
+    *last++ = '\n';
+
+    auto len = static_cast<size_t>(last - buf.data());
+    if (len + min_space > buf.size()) {
+      if (hexdump_write(fd, {buf.data(), len}) != 0) {
+        return -1;
+      }
+
+      last = buf.data();
+    }
   }
+
+  last = hexdump_addr(last, datalen);
+  *last++ = '\n';
+
+  auto len = static_cast<size_t>(last - buf.data());
+  if (len) {
+    return hexdump_write(fd, {buf.data(), len});
+  }
+
+  return 0;
 }
 
-std::string make_cid_key(const ngtcp2_cid *cid) {
-  return std::string(cid->data, cid->data + cid->datalen);
-}
+ngtcp2_cid make_cid_key(std::span<const uint8_t> cid) {
+  assert(cid.size() <= NGTCP2_MAX_CIDLEN);
 
-std::string make_cid_key(const uint8_t *cid, size_t cidlen) {
-  return std::string(cid, cid + cidlen);
+  ngtcp2_cid res;
+
+  std::ranges::copy(cid, std::begin(res.data));
+  res.datalen = cid.size();
+
+  return res;
 }
 
 std::string straddr(const sockaddr *sa, socklen_t salen) {
@@ -268,6 +377,30 @@ std::string straddr(const sockaddr *sa, socklen_t salen) {
   return res;
 }
 
+uint16_t port(const sockaddr_union *su) {
+  switch (su->sa.sa_family) {
+  case AF_INET:
+    return ntohs(su->in.sin_port);
+  case AF_INET6:
+    return ntohs(su->in6.sin6_port);
+  default:
+    return 0;
+  }
+}
+
+bool prohibited_port(uint16_t port) {
+  switch (port) {
+  case 1900:
+  case 5353:
+  case 11211:
+  case 20800:
+  case 27015:
+    return true;
+  default:
+    return port < 1024;
+  }
+}
+
 std::string_view strccalgo(ngtcp2_cc_algo cc_algo) {
   switch (cc_algo) {
   case NGTCP2_CC_ALGO_RENO:
@@ -276,8 +409,6 @@ std::string_view strccalgo(ngtcp2_cc_algo cc_algo) {
     return "cubic"sv;
   case NGTCP2_CC_ALGO_BBR:
     return "bbr"sv;
-  case NGTCP2_CC_ALGO_BBR2:
-    return "bbr2"sv;
   default:
     assert(0);
     abort();
@@ -587,14 +718,14 @@ int create_nonblock_socket(int domain, int type, int protocol) {
   if (fd == -1) {
     return -1;
   }
-#else  // !SOCK_NONBLOCK
+#else  // !defined(SOCK_NONBLOCK)
   auto fd = socket(domain, type, protocol);
   if (fd == -1) {
     return -1;
   }
 
   make_socket_nonblocking(fd);
-#endif // !SOCK_NONBLOCK
+#endif // !defined(SOCK_NONBLOCK)
 
   return fd;
 }
@@ -637,10 +768,31 @@ std::optional<uint32_t> parse_version(const std::string_view &s) {
   return v;
 }
 
+std::optional<std::string> read_token(const std::string_view &filename) {
+  return read_pem(filename, "token", "QUIC TOKEN");
+}
+
+int write_token(const std::string_view &filename,
+                std::span<const uint8_t> token) {
+  return write_pem(filename, "token", "QUIC TOKEN", token);
+}
+
+std::optional<std::string>
+read_transport_params(const std::string_view &filename) {
+  return read_pem(filename, "transport parameters",
+                  "QUIC TRANSPORT PARAMETERS");
+}
+
+int write_transport_params(const std::string_view &filename,
+                           std::span<const uint8_t> data) {
+  return write_pem(filename, "transport parameters",
+                   "QUIC TRANSPORT PARAMETERS", data);
+}
+
 } // namespace util
 
 std::ostream &operator<<(std::ostream &os, const ngtcp2_cid &cid) {
-  return os << "0x" << util::format_hex(cid.data, cid.datalen);
+  return os << "0x" << util::format_hex({cid.data, cid.datalen});
 }
 
 } // namespace ngtcp2
