@@ -54,6 +54,8 @@
 #include "ngtcp2_window_filter_test.h"
 #include "ngtcp2_settings_test.h"
 #include "ngtcp2_ppe_test.h"
+#include "ngtcp2_dcidtr_test.h"
+#include "ngtcp2_addr_test.h"
 
 int main(int argc, char *argv[]) {
   const MunitSuite suites[] = {
@@ -81,10 +83,14 @@ int main(int argc, char *argv[]) {
     window_filter_suite,
     settings_suite,
     ppe_suite,
-    {NULL, NULL, NULL, 0, MUNIT_SUITE_OPTION_NONE},
+    dcidtr_suite,
+    addr_suite,
+    {0},
   };
   const MunitSuite suite = {
-    "", NULL, suites, 1, MUNIT_SUITE_OPTION_NONE,
+    .prefix = "",
+    .suites = suites,
+    .iterations = 1,
   };
 
   init_static_path();
