@@ -35,11 +35,11 @@
 
 extern const MunitSuite ksl_suite;
 
-munit_void_test_decl(test_ngtcp2_ksl_insert);
-munit_void_test_decl(test_ngtcp2_ksl_clear);
-munit_void_test_decl(test_ngtcp2_ksl_range);
-munit_void_test_decl(test_ngtcp2_ksl_update_key_range);
-munit_void_test_decl(test_ngtcp2_ksl_dup);
-munit_void_test_decl(test_ngtcp2_ksl_remove_hint);
+munit_void_test_decl(test_ngtcp2_ksl_insert)
+munit_void_test_decl(test_ngtcp2_ksl_clear)
+munit_void_test_decl(test_ngtcp2_ksl_range)
+munit_void_test_decl(test_ngtcp2_ksl_update_key_range)
+munit_void_test_decl(test_ngtcp2_ksl_dup)
+munit_void_test_decl(test_ngtcp2_ksl_remove_hint)
 
 #endif /* !defined(NGTCP2_KSL_TEST_H) */
