@@ -27,12 +27,25 @@
 #include <stdio.h>
 #include <assert.h>
 
-#include <CUnit/CUnit.h>
-
 #include "ngtcp2_rtb.h"
 #include "ngtcp2_test_helper.h"
 #include "ngtcp2_mem.h"
 #include "ngtcp2_pkt.h"
+#include "ngtcp2_frame_chain.h"
+
+static const MunitTest tests[] = {
+  munit_void_test(test_ngtcp2_rtb_add),
+  munit_void_test(test_ngtcp2_rtb_recv_ack),
+  munit_void_test(test_ngtcp2_rtb_lost_pkt_ts),
+  munit_void_test(test_ngtcp2_rtb_remove_expired_lost_pkt),
+  munit_void_test(test_ngtcp2_rtb_remove_excessive_lost_pkt),
+  munit_test_end(),
+};
+
+const MunitSuite rtb_suite = {
+  .prefix = "/rtb",
+  .tests = tests,
+};
 
 static void conn_stat_init(ngtcp2_conn_stat *cstat) {
   memset(cstat, 0, sizeof(*cstat));
@@ -50,8 +63,6 @@ void test_ngtcp2_rtb_add(void) {
   ngtcp2_ksl_it it;
   ngtcp2_conn_stat cstat;
   ngtcp2_cc_reno cc;
-  ngtcp2_strm crypto;
-  const ngtcp2_pktns_id pktns_id = NGTCP2_PKTNS_ID_HANDSHAKE;
   ngtcp2_rst rst;
   ngtcp2_objalloc frc_objalloc;
   ngtcp2_objalloc rtb_entry_objalloc;
@@ -59,43 +70,41 @@ void test_ngtcp2_rtb_add(void) {
   ngtcp2_objalloc_init(&frc_objalloc, 1024, mem);
   ngtcp2_objalloc_init(&rtb_entry_objalloc, 1024, mem);
 
-  ngtcp2_strm_init(&crypto, 0, NGTCP2_STRM_FLAG_NONE, 0, 0, NULL, &frc_objalloc,
-                   mem);
   dcid_init(&dcid);
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
   ngtcp2_log_init(&log, NULL, NULL, 0, NULL);
   ngtcp2_cc_reno_init(&cc, &log);
-  ngtcp2_rtb_init(&rtb, pktns_id, &crypto, &rst, &cc.cc, 0, &log, NULL,
-                  &rtb_entry_objalloc, &frc_objalloc, mem);
+  ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
+                  &frc_objalloc, mem);
 
   ngtcp2_pkt_hd_init(&hd, NGTCP2_PKT_FLAG_NONE, NGTCP2_PKT_1RTT, &dcid, NULL,
-                     1000000007, 1, NGTCP2_PROTO_VER_V1, 0);
+                     1000000007, 1, NGTCP2_PROTO_VER_V1);
 
   rv = ngtcp2_rtb_entry_objalloc_new(
-      &ent, &hd, NULL, 10, 0, NGTCP2_RTB_ENTRY_FLAG_NONE, &rtb_entry_objalloc);
+    &ent, &hd, NULL, 10, 0, NGTCP2_RTB_ENTRY_FLAG_NONE, &rtb_entry_objalloc);
 
-  CU_ASSERT(0 == rv);
+  assert_int(0, ==, rv);
 
   ngtcp2_rtb_add(&rtb, ent, &cstat);
 
   ngtcp2_pkt_hd_init(&hd, NGTCP2_PKT_FLAG_NONE, NGTCP2_PKT_1RTT, &dcid, NULL,
-                     1000000008, 2, NGTCP2_PROTO_VER_V1, 0);
+                     1000000008, 2, NGTCP2_PROTO_VER_V1);
 
   rv = ngtcp2_rtb_entry_objalloc_new(
-      &ent, &hd, NULL, 9, 0, NGTCP2_RTB_ENTRY_FLAG_NONE, &rtb_entry_objalloc);
+    &ent, &hd, NULL, 9, 0, NGTCP2_RTB_ENTRY_FLAG_NONE, &rtb_entry_objalloc);
 
-  CU_ASSERT(0 == rv);
+  assert_int(0, ==, rv);
 
   ngtcp2_rtb_add(&rtb, ent, &cstat);
 
   ngtcp2_pkt_hd_init(&hd, NGTCP2_PKT_FLAG_NONE, NGTCP2_PKT_1RTT, &dcid, NULL,
-                     1000000009, 4, NGTCP2_PROTO_VER_V1, 0);
+                     1000000009, 4, NGTCP2_PROTO_VER_V1);
 
   rv = ngtcp2_rtb_entry_objalloc_new(
-      &ent, &hd, NULL, 11, 0, NGTCP2_RTB_ENTRY_FLAG_NONE, &rtb_entry_objalloc);
+    &ent, &hd, NULL, 11, 0, NGTCP2_RTB_ENTRY_FLAG_NONE, &rtb_entry_objalloc);
 
-  CU_ASSERT(0 == rv);
+  assert_int(0, ==, rv);
 
   ngtcp2_rtb_add(&rtb, ent, &cstat);
 
@@ -103,24 +112,23 @@ void test_ngtcp2_rtb_add(void) {
   ent = ngtcp2_ksl_it_get(&it);
 
   /* Check the top of the queue */
-  CU_ASSERT(1000000009 == ent->hd.pkt_num);
+  assert_int64(1000000009, ==, ent->hd.pkt_num);
 
   ngtcp2_ksl_it_next(&it);
   ent = ngtcp2_ksl_it_get(&it);
 
-  CU_ASSERT(1000000008 == ent->hd.pkt_num);
+  assert_int64(1000000008, ==, ent->hd.pkt_num);
 
   ngtcp2_ksl_it_next(&it);
   ent = ngtcp2_ksl_it_get(&it);
 
-  CU_ASSERT(1000000007 == ent->hd.pkt_num);
+  assert_int64(1000000007, ==, ent->hd.pkt_num);
 
   ngtcp2_ksl_it_next(&it);
 
-  CU_ASSERT(ngtcp2_ksl_it_end(&it));
+  assert_true(ngtcp2_ksl_it_end(&it));
 
   ngtcp2_rtb_free(&rtb);
-  ngtcp2_strm_free(&crypto);
 
   ngtcp2_objalloc_free(&rtb_entry_objalloc);
   ngtcp2_objalloc_free(&frc_objalloc);
@@ -138,7 +146,7 @@ static void add_rtb_entry_range(ngtcp2_rtb *rtb, int64_t base_pkt_num,
 
   for (i = 0; i < len; ++i) {
     ngtcp2_pkt_hd_init(&hd, NGTCP2_PKT_FLAG_NONE, NGTCP2_PKT_1RTT, &dcid, NULL,
-                       base_pkt_num + (int64_t)i, 1, NGTCP2_PROTO_VER_V1, 0);
+                       base_pkt_num + (int64_t)i, 1, NGTCP2_PROTO_VER_V1);
     ngtcp2_rtb_entry_objalloc_new(&ent, &hd, NULL, 0, 0,
                                   NGTCP2_RTB_ENTRY_FLAG_NONE, objalloc);
     ngtcp2_rtb_add(rtb, ent, cstat);
@@ -161,7 +169,7 @@ static void assert_rtb_entry_not_found(ngtcp2_rtb *rtb, int64_t pkt_num) {
 
   for (; !ngtcp2_ksl_it_end(&it); ngtcp2_ksl_it_next(&it)) {
     ent = ngtcp2_ksl_it_get(&it);
-    CU_ASSERT(ent->hd.pkt_num != pkt_num);
+    assert_int64(ent->hd.pkt_num, !=, pkt_num);
   }
 }
 
@@ -176,40 +184,38 @@ void test_ngtcp2_rtb_recv_ack(void) {
   ngtcp2_cc_reno cc;
   ngtcp2_pkt_hd hd;
   ngtcp2_ssize num_acked;
-  ngtcp2_strm crypto;
-  const ngtcp2_pktns_id pktns_id = NGTCP2_PKTNS_ID_HANDSHAKE;
   ngtcp2_rst rst;
   ngtcp2_objalloc frc_objalloc;
   ngtcp2_objalloc rtb_entry_objalloc;
+  ngtcp2_pktns pktns = {0};
 
   ngtcp2_objalloc_init(&frc_objalloc, 1024, mem);
   ngtcp2_objalloc_init(&rtb_entry_objalloc, 1024, mem);
 
-  ngtcp2_strm_init(&crypto, 0, NGTCP2_STRM_FLAG_NONE, 0, 0, NULL, &frc_objalloc,
-                   mem);
   ngtcp2_log_init(&log, NULL, NULL, 0, NULL);
   ngtcp2_pkt_hd_init(&hd, NGTCP2_PKT_FLAG_NONE, NGTCP2_PKT_1RTT, NULL, NULL, 0,
-                     1, NGTCP2_PROTO_VER_V1, 0);
+                     1, NGTCP2_PROTO_VER_V1);
+  pktns.id = NGTCP2_PKTNS_ID_HANDSHAKE;
 
   /* no ack block */
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
   ngtcp2_cc_reno_init(&cc, &log);
-  ngtcp2_rtb_init(&rtb, pktns_id, &crypto, &rst, &cc.cc, 0, &log, NULL,
-                  &rtb_entry_objalloc, &frc_objalloc, mem);
+  ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
+                  &frc_objalloc, mem);
   setup_rtb_fixture(&rtb, &cstat, &rtb_entry_objalloc);
 
-  CU_ASSERT(67 == ngtcp2_ksl_len(&rtb.ents));
+  assert_size(67, ==, ngtcp2_ksl_len(&rtb.ents));
 
   fr->largest_ack = 446;
   fr->first_ack_range = 1;
   fr->rangecnt = 0;
 
   num_acked =
-      ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, NULL, 1000000009, 1000000009);
+    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
 
-  CU_ASSERT(2 == num_acked);
-  CU_ASSERT(65 == ngtcp2_ksl_len(&rtb.ents));
+  assert_ptrdiff(2, ==, num_acked);
+  assert_size(65, ==, ngtcp2_ksl_len(&rtb.ents));
   assert_rtb_entry_not_found(&rtb, 446);
   assert_rtb_entry_not_found(&rtb, 445);
 
@@ -218,8 +224,8 @@ void test_ngtcp2_rtb_recv_ack(void) {
   /* with ack block */
   conn_stat_init(&cstat);
   ngtcp2_cc_reno_init(&cc, &log);
-  ngtcp2_rtb_init(&rtb, pktns_id, &crypto, &rst, &cc.cc, 0, &log, NULL,
-                  &rtb_entry_objalloc, &frc_objalloc, mem);
+  ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
+                  &frc_objalloc, mem);
   setup_rtb_fixture(&rtb, &cstat, &rtb_entry_objalloc);
 
   fr->largest_ack = 441;
@@ -232,11 +238,11 @@ void test_ngtcp2_rtb_recv_ack(void) {
   ranges[1].len = 1; /* (180), 179 */
 
   num_acked =
-      ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, NULL, 1000000009, 1000000009);
+    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
 
-  CU_ASSERT(4 == num_acked);
-  CU_ASSERT(63 == ngtcp2_ksl_len(&rtb.ents));
-  CU_ASSERT(441 == rtb.largest_acked_tx_pkt_num);
+  assert_ptrdiff(4, ==, num_acked);
+  assert_size(63, ==, ngtcp2_ksl_len(&rtb.ents));
+  assert_int64(441, ==, rtb.largest_acked_tx_pkt_num);
   assert_rtb_entry_not_found(&rtb, 441);
   assert_rtb_entry_not_found(&rtb, 440);
   assert_rtb_entry_not_found(&rtb, 183);
@@ -247,8 +253,8 @@ void test_ngtcp2_rtb_recv_ack(void) {
   /* gap+len points to pkt_num 0 */
   conn_stat_init(&cstat);
   ngtcp2_cc_reno_init(&cc, &log);
-  ngtcp2_rtb_init(&rtb, pktns_id, &crypto, &rst, &cc.cc, 0, &log, NULL,
-                  &rtb_entry_objalloc, &frc_objalloc, mem);
+  ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
+                  &frc_objalloc, mem);
   add_rtb_entry_range(&rtb, 0, 1, &cstat, &rtb_entry_objalloc);
 
   fr->largest_ack = 250;
@@ -258,9 +264,9 @@ void test_ngtcp2_rtb_recv_ack(void) {
   fr->ranges[0].len = 0;
 
   num_acked =
-      ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, NULL, 1000000009, 1000000009);
+    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
 
-  CU_ASSERT(1 == num_acked);
+  assert_ptrdiff(1, ==, num_acked);
   assert_rtb_entry_not_found(&rtb, 0);
 
   ngtcp2_rtb_free(&rtb);
@@ -268,8 +274,8 @@ void test_ngtcp2_rtb_recv_ack(void) {
   /* pkt_num = 0 (first ack block) */
   conn_stat_init(&cstat);
   ngtcp2_cc_reno_init(&cc, &log);
-  ngtcp2_rtb_init(&rtb, pktns_id, &crypto, &rst, &cc.cc, 0, &log, NULL,
-                  &rtb_entry_objalloc, &frc_objalloc, mem);
+  ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
+                  &frc_objalloc, mem);
   add_rtb_entry_range(&rtb, 0, 1, &cstat, &rtb_entry_objalloc);
 
   fr->largest_ack = 0;
@@ -277,9 +283,9 @@ void test_ngtcp2_rtb_recv_ack(void) {
   fr->rangecnt = 0;
 
   num_acked =
-      ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, NULL, 1000000009, 1000000009);
+    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
 
-  CU_ASSERT(1 == num_acked);
+  assert_ptrdiff(1, ==, num_acked);
   assert_rtb_entry_not_found(&rtb, 0);
 
   ngtcp2_rtb_free(&rtb);
@@ -287,8 +293,8 @@ void test_ngtcp2_rtb_recv_ack(void) {
   /* pkt_num = 0 */
   conn_stat_init(&cstat);
   ngtcp2_cc_reno_init(&cc, &log);
-  ngtcp2_rtb_init(&rtb, pktns_id, &crypto, &rst, &cc.cc, 0, &log, NULL,
-                  &rtb_entry_objalloc, &frc_objalloc, mem);
+  ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
+                  &frc_objalloc, mem);
   add_rtb_entry_range(&rtb, 0, 1, &cstat, &rtb_entry_objalloc);
 
   fr->largest_ack = 2;
@@ -298,13 +304,12 @@ void test_ngtcp2_rtb_recv_ack(void) {
   fr->ranges[0].len = 0;
 
   num_acked =
-      ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, NULL, 1000000009, 1000000009);
+    ngtcp2_rtb_recv_ack(&rtb, fr, &cstat, NULL, &pktns, 1000000009, 1000000009);
 
-  CU_ASSERT(1 == num_acked);
+  assert_ptrdiff(1, ==, num_acked);
   assert_rtb_entry_not_found(&rtb, 0);
 
   ngtcp2_rtb_free(&rtb);
-  ngtcp2_strm_free(&crypto);
 
   ngtcp2_objalloc_free(&rtb_entry_objalloc);
   ngtcp2_objalloc_free(&frc_objalloc);
@@ -312,8 +317,6 @@ void test_ngtcp2_rtb_recv_ack(void) {
 
 void test_ngtcp2_rtb_lost_pkt_ts(void) {
   ngtcp2_rtb rtb;
-  const ngtcp2_pktns_id pktns_id = NGTCP2_PKTNS_ID_APPLICATION;
-  ngtcp2_strm crypto;
   ngtcp2_log log;
   const ngtcp2_mem *mem = ngtcp2_mem_default();
   ngtcp2_cc_reno cc;
@@ -327,19 +330,17 @@ void test_ngtcp2_rtb_lost_pkt_ts(void) {
   ngtcp2_objalloc_init(&frc_objalloc, 1024, mem);
   ngtcp2_objalloc_init(&rtb_entry_objalloc, 1024, mem);
 
-  ngtcp2_strm_init(&crypto, 0, NGTCP2_STRM_FLAG_NONE, 0, 0, NULL, &frc_objalloc,
-                   mem);
   ngtcp2_log_init(&log, NULL, NULL, 0, NULL);
 
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
   ngtcp2_cc_reno_init(&cc, &log);
-  ngtcp2_rtb_init(&rtb, pktns_id, &crypto, &rst, &cc.cc, 0, &log, NULL,
-                  &rtb_entry_objalloc, &frc_objalloc, mem);
+  ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
+                  &frc_objalloc, mem);
 
   add_rtb_entry_range(&rtb, 0, 1, &cstat, &rtb_entry_objalloc);
 
-  CU_ASSERT(UINT64_MAX == ngtcp2_rtb_lost_pkt_ts(&rtb));
+  assert_uint64(UINT64_MAX, ==, ngtcp2_rtb_lost_pkt_ts(&rtb));
 
   it = ngtcp2_ksl_end(&rtb.ents);
   ngtcp2_ksl_it_prev(&it);
@@ -347,10 +348,9 @@ void test_ngtcp2_rtb_lost_pkt_ts(void) {
   ent->flags |= NGTCP2_RTB_ENTRY_FLAG_LOST_RETRANSMITTED;
   ent->lost_ts = 16777217;
 
-  CU_ASSERT(16777217 == ngtcp2_rtb_lost_pkt_ts(&rtb));
+  assert_uint64(16777217, ==, ngtcp2_rtb_lost_pkt_ts(&rtb));
 
   ngtcp2_rtb_free(&rtb);
-  ngtcp2_strm_free(&crypto);
 
   ngtcp2_objalloc_free(&rtb_entry_objalloc);
   ngtcp2_objalloc_free(&frc_objalloc);
@@ -358,8 +358,6 @@ void test_ngtcp2_rtb_lost_pkt_ts(void) {
 
 void test_ngtcp2_rtb_remove_expired_lost_pkt(void) {
   ngtcp2_rtb rtb;
-  const ngtcp2_pktns_id pktns_id = NGTCP2_PKTNS_ID_APPLICATION;
-  ngtcp2_strm crypto;
   ngtcp2_log log;
   const ngtcp2_mem *mem = ngtcp2_mem_default();
   ngtcp2_cc_reno cc;
@@ -374,15 +372,13 @@ void test_ngtcp2_rtb_remove_expired_lost_pkt(void) {
   ngtcp2_objalloc_init(&frc_objalloc, 1024, mem);
   ngtcp2_objalloc_init(&rtb_entry_objalloc, 1024, mem);
 
-  ngtcp2_strm_init(&crypto, 0, NGTCP2_STRM_FLAG_NONE, 0, 0, NULL, &frc_objalloc,
-                   mem);
   ngtcp2_log_init(&log, NULL, NULL, 0, NULL);
 
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
   ngtcp2_cc_reno_init(&cc, &log);
-  ngtcp2_rtb_init(&rtb, pktns_id, &crypto, &rst, &cc.cc, 0, &log, NULL,
-                  &rtb_entry_objalloc, &frc_objalloc, mem);
+  ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
+                  &frc_objalloc, mem);
 
   add_rtb_entry_range(&rtb, 0, 7, &cstat, &rtb_entry_objalloc);
 
@@ -397,14 +393,13 @@ void test_ngtcp2_rtb_remove_expired_lost_pkt(void) {
 
   ngtcp2_rtb_remove_expired_lost_pkt(&rtb, 1, 16777219);
 
-  CU_ASSERT(5 == ngtcp2_ksl_len(&rtb.ents));
+  assert_size(5, ==, ngtcp2_ksl_len(&rtb.ents));
 
   ngtcp2_rtb_remove_expired_lost_pkt(&rtb, 1, 16777223);
 
-  CU_ASSERT(2 == ngtcp2_ksl_len(&rtb.ents));
+  assert_size(2, ==, ngtcp2_ksl_len(&rtb.ents));
 
   ngtcp2_rtb_free(&rtb);
-  ngtcp2_strm_free(&crypto);
 
   ngtcp2_objalloc_free(&rtb_entry_objalloc);
   ngtcp2_objalloc_free(&frc_objalloc);
@@ -412,8 +407,6 @@ void test_ngtcp2_rtb_remove_expired_lost_pkt(void) {
 
 void test_ngtcp2_rtb_remove_excessive_lost_pkt(void) {
   ngtcp2_rtb rtb;
-  const ngtcp2_pktns_id pktns_id = NGTCP2_PKTNS_ID_APPLICATION;
-  ngtcp2_strm crypto;
   ngtcp2_log log;
   const ngtcp2_mem *mem = ngtcp2_mem_default();
   ngtcp2_cc_reno cc;
@@ -428,15 +421,13 @@ void test_ngtcp2_rtb_remove_excessive_lost_pkt(void) {
   ngtcp2_objalloc_init(&frc_objalloc, 1024, mem);
   ngtcp2_objalloc_init(&rtb_entry_objalloc, 1024, mem);
 
-  ngtcp2_strm_init(&crypto, 0, NGTCP2_STRM_FLAG_NONE, 0, 0, NULL, &frc_objalloc,
-                   mem);
   ngtcp2_log_init(&log, NULL, NULL, 0, NULL);
 
   conn_stat_init(&cstat);
   ngtcp2_rst_init(&rst);
   ngtcp2_cc_reno_init(&cc, &log);
-  ngtcp2_rtb_init(&rtb, pktns_id, &crypto, &rst, &cc.cc, 0, &log, NULL,
-                  &rtb_entry_objalloc, &frc_objalloc, mem);
+  ngtcp2_rtb_init(&rtb, &rst, &cc.cc, 0, &log, NULL, &rtb_entry_objalloc,
+                  &frc_objalloc, mem);
 
   add_rtb_entry_range(&rtb, 0, 7, &cstat, &rtb_entry_objalloc);
 
@@ -452,10 +443,9 @@ void test_ngtcp2_rtb_remove_excessive_lost_pkt(void) {
 
   ngtcp2_rtb_remove_excessive_lost_pkt(&rtb, 2);
 
-  CU_ASSERT(4 == ngtcp2_ksl_len(&rtb.ents));
+  assert_size(4, ==, ngtcp2_ksl_len(&rtb.ents));
 
   ngtcp2_rtb_free(&rtb);
-  ngtcp2_strm_free(&crypto);
 
   ngtcp2_objalloc_free(&rtb_entry_objalloc);
   ngtcp2_objalloc_free(&frc_objalloc);
