@@ -26,11 +26,23 @@
 
 #include <stdio.h>
 
-#include <CUnit/CUnit.h>
-
 #include "ngtcp2_strm.h"
 #include "ngtcp2_test_helper.h"
 #include "ngtcp2_vec.h"
+#include "ngtcp2_frame_chain.h"
+
+static const MunitTest tests[] = {
+  munit_void_test(test_ngtcp2_strm_streamfrq_pop),
+  munit_void_test(test_ngtcp2_strm_streamfrq_unacked_offset),
+  munit_void_test(test_ngtcp2_strm_streamfrq_unacked_pop),
+  munit_void_test(test_ngtcp2_strm_discard_reordered_data),
+  munit_test_end(),
+};
+
+const MunitSuite strm_suite = {
+  .prefix = "/strm",
+  .tests = tests,
+};
 
 static uint8_t nulldata[1024];
 
@@ -90,6 +102,8 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   int rv;
   ngtcp2_vec *data;
   ngtcp2_objalloc frc_objalloc;
+  size_t i;
+  ngtcp2_ksl_it it;
 
   ngtcp2_objalloc_init(&frc_objalloc, 1024, mem);
 
@@ -99,14 +113,14 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 30);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(2 == frc->fr.stream.datacnt);
+  assert_int(0, ==, rv);
+  assert_size(2, ==, frc->fr.stream.datacnt);
 
   data = frc->fr.stream.data;
 
-  CU_ASSERT(11 == data[0].len);
-  CU_ASSERT(19 == data[1].len);
-  CU_ASSERT(2 == ngtcp2_ksl_len(strm.tx.streamfrq));
+  assert_size(11, ==, data[0].len);
+  assert_size(19, ==, data[1].len);
+  assert_size(2, ==, ngtcp2_ksl_len(strm.tx.streamfrq));
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
@@ -117,14 +131,14 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 76);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(2 == frc->fr.stream.datacnt);
+  assert_int(0, ==, rv);
+  assert_size(2, ==, frc->fr.stream.datacnt);
 
   data = frc->fr.stream.data;
 
-  CU_ASSERT(11 == data[0].len);
-  CU_ASSERT(19 + 46 == data[1].len);
-  CU_ASSERT(1 == ngtcp2_ksl_len(strm.tx.streamfrq));
+  assert_size(11, ==, data[0].len);
+  assert_size(19 + 46, ==, data[1].len);
+  assert_size(1, ==, ngtcp2_ksl_len(strm.tx.streamfrq));
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
@@ -135,25 +149,25 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 75);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(2 == frc->fr.stream.datacnt);
+  assert_int(0, ==, rv);
+  assert_size(2, ==, frc->fr.stream.datacnt);
 
   data = frc->fr.stream.data;
 
-  CU_ASSERT(11 == data[0].len);
-  CU_ASSERT(19 + 45 == data[1].len);
-  CU_ASSERT(2 == ngtcp2_ksl_len(strm.tx.streamfrq));
+  assert_size(11, ==, data[0].len);
+  assert_size(19 + 45, ==, data[1].len);
+  assert_size(2, ==, ngtcp2_ksl_len(strm.tx.streamfrq));
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
 
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 1);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(75 == frc->fr.stream.offset);
-  CU_ASSERT(1 == frc->fr.stream.datacnt);
-  CU_ASSERT(1 == frc->fr.stream.data[0].len);
-  CU_ASSERT(nulldata + 30 + 17 + 28 == frc->fr.stream.data[0].base);
+  assert_int(0, ==, rv);
+  assert_uint64(75, ==, frc->fr.stream.offset);
+  assert_size(1, ==, frc->fr.stream.datacnt);
+  assert_size(1, ==, frc->fr.stream.data[0].len);
+  assert_ptr_equal(nulldata + 30 + 17 + 28, frc->fr.stream.data[0].base);
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
@@ -164,30 +178,30 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 77);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(3 == frc->fr.stream.datacnt);
+  assert_int(0, ==, rv);
+  assert_size(3, ==, frc->fr.stream.datacnt);
 
   data = frc->fr.stream.data;
 
-  CU_ASSERT(11 == data[0].len);
-  CU_ASSERT(19 + 46 == data[1].len);
-  CU_ASSERT(1 == data[2].len);
-  CU_ASSERT(nulldata + 256 == data[2].base);
-  CU_ASSERT(1 == ngtcp2_ksl_len(strm.tx.streamfrq));
+  assert_size(11, ==, data[0].len);
+  assert_size(19 + 46, ==, data[1].len);
+  assert_size(1, ==, data[2].len);
+  assert_ptr_equal(nulldata + 256, data[2].base);
+  assert_size(1, ==, ngtcp2_ksl_len(strm.tx.streamfrq));
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
 
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 1024);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(77 == frc->fr.stream.offset);
-  CU_ASSERT(2 == frc->fr.stream.datacnt);
+  assert_int(0, ==, rv);
+  assert_uint64(77, ==, frc->fr.stream.offset);
+  assert_size(2, ==, frc->fr.stream.datacnt);
 
   data = frc->fr.stream.data;
 
-  CU_ASSERT(30 == data[0].len);
-  CU_ASSERT(nulldata + 256 + 1 == data[0].base);
+  assert_size(30, ==, data[0].len);
+  assert_ptr_equal(nulldata + 256 + 1, data[0].base);
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
@@ -198,34 +212,34 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 12);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(0 == frc->fr.stream.offset);
-  CU_ASSERT(2 == frc->fr.stream.datacnt);
+  assert_int(0, ==, rv);
+  assert_uint64(0, ==, frc->fr.stream.offset);
+  assert_size(2, ==, frc->fr.stream.datacnt);
 
   data = frc->fr.stream.data;
 
-  CU_ASSERT(11 == data[0].len);
-  CU_ASSERT(nulldata == data[0].base);
-  CU_ASSERT(1 == data[1].len);
-  CU_ASSERT(nulldata + 11 == data[1].base);
+  assert_size(11, ==, data[0].len);
+  assert_ptr_equal(nulldata, data[0].base);
+  assert_size(1, ==, data[1].len);
+  assert_ptr_equal(nulldata + 11, data[1].base);
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
 
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 1024);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(12 == frc->fr.stream.offset);
-  CU_ASSERT(3 == frc->fr.stream.datacnt);
+  assert_int(0, ==, rv);
+  assert_uint64(12, ==, frc->fr.stream.offset);
+  assert_size(3, ==, frc->fr.stream.datacnt);
 
   data = frc->fr.stream.data;
 
-  CU_ASSERT(64 == data[0].len);
-  CU_ASSERT(nulldata + 12 == data[0].base);
-  CU_ASSERT(31 == data[1].len);
-  CU_ASSERT(nulldata + 256 == data[1].base);
-  CU_ASSERT(1 == data[2].len);
-  CU_ASSERT(nulldata + 512 == data[2].base);
+  assert_size(64, ==, data[0].len);
+  assert_ptr_equal(nulldata + 12, data[0].base);
+  assert_size(31, ==, data[1].len);
+  assert_ptr_equal(nulldata + 256, data[1].base);
+  assert_size(1, ==, data[2].len);
+  assert_ptr_equal(nulldata + 512, data[2].base);
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
@@ -259,10 +273,10 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 1024);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(1 == frc->fr.stream.datacnt);
-  CU_ASSERT(11 == frc->fr.stream.data[0].len);
-  CU_ASSERT(1 == ngtcp2_ksl_len(strm.tx.streamfrq));
+  assert_int(0, ==, rv);
+  assert_size(1, ==, frc->fr.stream.datacnt);
+  assert_size(11, ==, frc->fr.stream.data[0].len);
+  assert_size(1, ==, ngtcp2_ksl_len(strm.tx.streamfrq));
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
@@ -293,9 +307,9 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 1024);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(1 == frc->fr.stream.fin);
-  CU_ASSERT(1 == frc->fr.stream.datacnt);
+  assert_int(0, ==, rv);
+  assert_true(frc->fr.stream.fin);
+  assert_size(1, ==, frc->fr.stream.datacnt);
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
@@ -306,8 +320,56 @@ void test_ngtcp2_strm_streamfrq_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 0);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(NULL == frc);
+  assert_int(0, ==, rv);
+  assert_null(frc);
+
+  ngtcp2_strm_free(&strm);
+
+  /* stream datacnt gets below the allocation threshold */
+  ngtcp2_strm_init(&strm, 0, NGTCP2_STRM_FLAG_NONE, 0, 0, NULL, &frc_objalloc,
+                   mem);
+  ngtcp2_frame_chain_stream_datacnt_objalloc_new(&frc, 1, &frc_objalloc, mem);
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.offset = 0;
+  frc->fr.stream.datacnt = 1;
+  data = frc->fr.stream.data;
+  data[0].len = 17;
+  data[0].base = nulldata;
+
+  ngtcp2_strm_streamfrq_push(&strm, frc);
+
+  ngtcp2_frame_chain_stream_datacnt_objalloc_new(
+    &frc, NGTCP2_FRAME_CHAIN_STREAM_DATACNT_THRES + 1, &frc_objalloc, mem);
+  frc->fr.stream.type = NGTCP2_FRAME_STREAM;
+  frc->fr.stream.fin = 0;
+  frc->fr.stream.offset = 17;
+  frc->fr.stream.datacnt = NGTCP2_FRAME_CHAIN_STREAM_DATACNT_THRES + 1;
+  data = frc->fr.stream.data;
+
+  for (i = 0; i < frc->fr.stream.datacnt; ++i) {
+    data[i].len = 1;
+    data[i].base = nulldata;
+  }
+
+  ngtcp2_strm_streamfrq_push(&strm, frc);
+
+  frc = NULL;
+  rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 18);
+
+  assert_int(0, ==, rv);
+  assert_false(frc->fr.stream.fin);
+  assert_size(2, ==, frc->fr.stream.datacnt);
+
+  ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
+
+  it = ngtcp2_ksl_begin(strm.tx.streamfrq);
+  frc = ngtcp2_ksl_it_get(&it);
+
+  assert_false(frc->fr.stream.fin);
+  assert_uint64(18, ==, frc->fr.stream.offset);
+  assert_size(NGTCP2_FRAME_CHAIN_STREAM_DATACNT_THRES, ==,
+              frc->fr.stream.datacnt);
 
   ngtcp2_strm_free(&strm);
 
@@ -351,7 +413,7 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
 
   ngtcp2_strm_ack_data(&strm, 0, 443 + 971);
 
-  CU_ASSERT((uint64_t)-1 == ngtcp2_strm_streamfrq_unacked_offset(&strm));
+  assert_uint64((uint64_t)-1, ==, ngtcp2_strm_streamfrq_unacked_offset(&strm));
 
   ngtcp2_strm_free(&strm);
 
@@ -383,7 +445,7 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
 
   ngtcp2_strm_ack_data(&strm, 0, 443 + 971);
 
-  CU_ASSERT(443 + 971 == ngtcp2_strm_streamfrq_unacked_offset(&strm));
+  assert_uint64(443 + 971, ==, ngtcp2_strm_streamfrq_unacked_offset(&strm));
 
   ngtcp2_strm_free(&strm);
 
@@ -404,7 +466,7 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
 
   ngtcp2_strm_ack_data(&strm, 0, 443);
 
-  CU_ASSERT(443 == ngtcp2_strm_streamfrq_unacked_offset(&strm));
+  assert_uint64(443, ==, ngtcp2_strm_streamfrq_unacked_offset(&strm));
 
   ngtcp2_strm_free(&strm);
 
@@ -425,7 +487,7 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
 
   ngtcp2_strm_ack_data(&strm, 0, 971);
 
-  CU_ASSERT((uint64_t)-1 == ngtcp2_strm_streamfrq_unacked_offset(&strm));
+  assert_uint64((uint64_t)-1, ==, ngtcp2_strm_streamfrq_unacked_offset(&strm));
 
   ngtcp2_strm_free(&strm);
 
@@ -446,7 +508,7 @@ void test_ngtcp2_strm_streamfrq_unacked_offset(void) {
 
   ngtcp2_strm_ack_data(&strm, 0, 971);
 
-  CU_ASSERT(977 == ngtcp2_strm_streamfrq_unacked_offset(&strm));
+  assert_uint64(977, ==, ngtcp2_strm_streamfrq_unacked_offset(&strm));
 
   ngtcp2_strm_free(&strm);
 
@@ -494,8 +556,8 @@ void test_ngtcp2_strm_streamfrq_unacked_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 1024);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(NULL == frc);
+  assert_int(0, ==, rv);
+  assert_null(frc);
 
   ngtcp2_strm_free(&strm);
 
@@ -530,11 +592,12 @@ void test_ngtcp2_strm_streamfrq_unacked_pop(void) {
   frc = NULL;
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 1024);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(NGTCP2_FRAME_STREAM == frc->fr.type);
-  CU_ASSERT(1 == frc->fr.stream.fin);
-  CU_ASSERT(764 == frc->fr.stream.offset);
-  CU_ASSERT(0 == ngtcp2_vec_len(frc->fr.stream.data, frc->fr.stream.datacnt));
+  assert_int(0, ==, rv);
+  assert_uint64(NGTCP2_FRAME_STREAM, ==, frc->fr.type);
+  assert_true(frc->fr.stream.fin);
+  assert_uint64(764, ==, frc->fr.stream.offset);
+  assert_uint64(0, ==,
+                ngtcp2_vec_len(frc->fr.stream.data, frc->fr.stream.datacnt));
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
@@ -546,12 +609,13 @@ void test_ngtcp2_strm_streamfrq_unacked_pop(void) {
 
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 43);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(NGTCP2_FRAME_STREAM == frc->fr.type);
-  CU_ASSERT(0 == frc->fr.stream.fin);
-  CU_ASSERT(12 == frc->fr.stream.offset);
-  CU_ASSERT(1 == frc->fr.stream.datacnt);
-  CU_ASSERT(43 == ngtcp2_vec_len(frc->fr.stream.data, frc->fr.stream.datacnt));
+  assert_int(0, ==, rv);
+  assert_uint64(NGTCP2_FRAME_STREAM, ==, frc->fr.type);
+  assert_false(frc->fr.stream.fin);
+  assert_uint64(12, ==, frc->fr.stream.offset);
+  assert_size(1, ==, frc->fr.stream.datacnt);
+  assert_uint64(43, ==,
+                ngtcp2_vec_len(frc->fr.stream.data, frc->fr.stream.datacnt));
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
@@ -563,15 +627,48 @@ void test_ngtcp2_strm_streamfrq_unacked_pop(void) {
 
   rv = ngtcp2_strm_streamfrq_pop(&strm, &frc, 43);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(NGTCP2_FRAME_STREAM == frc->fr.type);
-  CU_ASSERT(0 == frc->fr.stream.fin);
-  CU_ASSERT(0 == frc->fr.stream.offset);
-  CU_ASSERT(2 == frc->fr.stream.datacnt);
-  CU_ASSERT(32 == ngtcp2_vec_len(frc->fr.stream.data, frc->fr.stream.datacnt));
+  assert_int(0, ==, rv);
+  assert_uint64(NGTCP2_FRAME_STREAM, ==, frc->fr.type);
+  assert_false(frc->fr.stream.fin);
+  assert_uint64(0, ==, frc->fr.stream.offset);
+  assert_size(2, ==, frc->fr.stream.datacnt);
+  assert_uint64(32, ==,
+                ngtcp2_vec_len(frc->fr.stream.data, frc->fr.stream.datacnt));
 
   ngtcp2_frame_chain_objalloc_del(frc, &frc_objalloc, mem);
   ngtcp2_strm_free(&strm);
 
   ngtcp2_objalloc_free(&frc_objalloc);
 }
+
+void test_ngtcp2_strm_discard_reordered_data(void) {
+  ngtcp2_strm strm;
+  const ngtcp2_mem *mem = ngtcp2_mem_default();
+
+  /* No reordered data has been received. */
+  ngtcp2_strm_init(&strm, 0, NGTCP2_STRM_FLAG_NONE, 0, 0, NULL, NULL, mem);
+
+  ngtcp2_strm_update_rx_offset(&strm, 1000000007);
+  ngtcp2_strm_discard_reordered_data(&strm);
+
+  assert_null(strm.rx.rob);
+  assert_uint64(1000000007, ==, ngtcp2_strm_rx_offset(&strm));
+
+  ngtcp2_strm_free(&strm);
+
+  /* Discard reordered data */
+  ngtcp2_strm_init(&strm, 0, NGTCP2_STRM_FLAG_NONE, 0, 0, NULL, NULL, mem);
+
+  ngtcp2_strm_update_rx_offset(&strm, 1000000007);
+  ngtcp2_strm_recv_reordering(&strm, nulldata, 117, 1000000008);
+
+  assert_not_null(strm.rx.rob);
+  assert_uint64(1000000007, ==, ngtcp2_strm_rx_offset(&strm));
+
+  ngtcp2_strm_discard_reordered_data(&strm);
+
+  assert_null(strm.rx.rob);
+  assert_uint64(1000000007, ==, ngtcp2_strm_rx_offset(&strm));
+
+  ngtcp2_strm_free(&strm);
+}
